#include <gearlib.h>
#include <GLFW/glfw3.h>
#include <log.c/src/log.h>

i32 gr_window_init(i32 major, i32 minor) {
    if(gr_state->window_initialized == true) {
        log_fatal("Init already called");
        return 1;
    }

    gr_state_set(major, major);
    gr_state_set(minor, minor);
    gr_state_set(window_initialized, true);

    log_info("Initialized for OpenGL %d.%d", major, minor);

    return 0;
}

i32 gr_window_create(i32 width, i32 height, char* title) {
    if(!gr_state_get(window_initialized)) {
        log_info("Window was not initialized before calling create, defaulting to OpenGL %d.%d", 
                 GR_DEFAULT_MAJOR, GR_DEFAULT_MINOR);
        gr_window_init(GR_DEFAULT_MAJOR, GR_DEFAULT_MINOR);
    }

#ifdef USE_GLFW
    if(glfwInit() != GLFW_TRUE) {
        log_fatal("GLFW failed to initialize");
    } else {
        log_info("GLFW initialized successfully");
    }

    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, gr_state_get(major));
    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, gr_state_get(minor));
    glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
    glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);

    gr_state_set(window, glfwCreateWindow(width, height, title, NULL, NULL));
    if(gr_state_get(window) == NULL) {
        log_fatal("Failed to create window");
        return 1;
    } else {
        log_info("Window created successfully");
    }
#endif

    return 0;
}
